import { QueryOrderMap, Loaded, EntityManager, EntityName, FilterQuery, FindOptions } from '@mikro-orm/core';

type AfterCursorPaginationParams = {
    first: number;
    after: string;
};
type BeforeCursorPaginationParams = {
    last: number;
    before: string;
};
type NoCursorPaginationParams<T extends object> = {
    first: number;
    /**
     * Order by has been moved here (instead of being in native MikroORM options)
     * because it is required for the pagination to work when doing an initial query without cursor.
     *
     * To keep pagination consistent in case of entities having the same value on a column,
     * a unique key must be included in the order by.
     * Most of the time, using the primary key is a good choice.
     *
     * @example
     * // Ensure at least one unique column is used
     * cursorPaginationFind(em, User, {
     *   first: 10,
     *   orderBy: {
     *     id: 'ASC'
     *   }
     * })
     *
     * // When non uniques order by columns are used, always use another unique column as last order by.
     * cursorPaginationFind(em, User, {
     *   first: 10,
     *   orderBy: {
     *     firstName: 'DESC', // Multiple users can have the same first name
     *     lastName: 'DESC', // Multiple users can have the same last name
     *     id: 'ASC' // id is unique and will be used to keep pagination consistent
     *   }
     * })
     */
    orderBy: QueryOrderMap<T> | QueryOrderMap<T>[];
};
type PaginationParams<T extends object> = AfterCursorPaginationParams | BeforeCursorPaginationParams | NoCursorPaginationParams<T>;
type CursorPaginationResult<T extends object, P extends string> = {
    totalCount: number;
    edges: {
        cursor: string;
        node: Loaded<T, P>;
    }[];
    pageInfo: {
        hasNextPage: boolean;
        hasPreviousPage: boolean;
        startCursor?: string;
        endCursor?: string;
    };
};
/**
 * Cursor-based pagination with support for multiple order by columns.
 *
 * @param em - MikroORM EntityManager.
 * @param entityName - Entity name.
 * @param pagination - Cursor pagination parameters.
 * @param where - Native MikroORM `find` filter query.
 * @param options - Native MikroORM `find` options without `limit`, `offset` and `orderBy` properties.
 *
 * @example
 *
 * ```ts
 * \@Entity()
 * class User {
 *   \@PrimaryKey()
 *   id!: number;
 *
 *   \@Property()
 *   name!: string;
 *
 *   \@Property()
 *   age!: number;
 *
 *   \@ManyToMany()
 *   friends = new Collection<User>(this);
 * }
 *
 * const page = await cursorPaginationFind(
 *   em,
 *   User,
 *   {
 *     first: 10,
 *     orderBy: {
 *       name: 'ASC',
 *       age: 'DESC',
 *       id: 'ASC',
 *     },
 *   },
 *   {
 *     age: { $gt: 20 },
 *   },
 *   {
 *     populate: ['friends'],
 *   }
 * );
 *
 * if (page.pageInfo.hasNextPage) {
 *   const nextPage = await cursorPaginationFind(
 *     em,
 *     User,
 *     {
 *       first: 10,
 *       after: page.pageInfo.endCursor,
 *     },
 *     {
 *       age: { $gt: 20 },
 *     },
 *     {
 *       populate: ['friends'],
 *     }
 *   );
 * }
 * ```
 */
declare function cursorPaginationFind<T extends object, P extends string = never>(em: EntityManager, entityName: EntityName<T>, pagination: PaginationParams<T>, where?: FilterQuery<T>, options?: Omit<FindOptions<T, P>, 'limit' | 'offset' | 'orderBy'>): Promise<CursorPaginationResult<T, P>>;

export { type AfterCursorPaginationParams, type BeforeCursorPaginationParams, type CursorPaginationResult, type NoCursorPaginationParams, type PaginationParams, cursorPaginationFind };
